library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity er_1octet is
  port ( rst : in std_logic ;
         clk : in std_logic ;
         en : in std_logic ;
         din : in std_logic_vector (7 downto 0) ;
         miso : in std_logic ;
         sclk : out std_logic ;
         mosi : out std_logic ;
         dout : out std_logic_vector (7 downto 0) ;
         busy : out std_logic);
end er_1octet;

architecture behavioral_3 of er_1octet is

	-- 00 -> Idle ; 01 -> bit_emis ; 10 -> bit_receptionne
	signal etat : std_logic_vector(1 downto 0);
	
begin

	process(clk, rst)
	
	-- registre temporaire
	variable reg : std_logic_vector(7 downto 0);
	
	-- Compteur permettant de savoir à quel bit on est
	variable cpt : natural;
	
	begin
	
		-- Reset
		if rst = '0' then
			-- Réinitialisation des variables et signaux
			-- L'État
			etat <= "00";
			-- La clock
			sclk <= '1';
			-- Le compteur
			cpt := 7;
			-- Le registre
			reg := (others => 'U');
			-- L'indicateur de travail
			busy <= '0';
			
			
		-- Sur front montant de la clock
		elsif(rising_edge(clk)) then
		
			-- Transition de l'état idle à bit_emis
			if etat = "00" and en = '1' then
				-- Mettre etat à bit_emis
				etat <= "01";
				-- prendre la valeur d'entrée
				reg := din;
				-- Initialiser le compteur
				cpt := 7;
				-- Envoyer l'octet de poids fort
				mosi <= reg(cpt);
				-- Signaler qu'on est occupé
				busy <= '1';
				-- Initialiser la clock
				sclk <= '0';
				
			-- Transition de l'état bit_emis à bit receptionne
			elsif etat = "01" then
				-- Mettre etat à bit_receptionne
				etat <= "10";
				-- Mettre clock à 1
				sclk <= '1';
				-- Réceptionner un bit
				reg(cpt) := miso;
				
			elsif etat = "10" then
				-- Transition de l'état bit_receptionne à bit_emis
				if cpt > 0 then
					-- Mettre etat à bit_emis
					etat <= "01";
					-- Décrémenter le compteur
					cpt := cpt - 1;
					-- Envoyer un bit
					mosi <= reg(cpt);
					-- Mettre clock à 0
					sclk <= '0';
					
				-- Transition de l'état bit_receptionne à idle
				elsif cpt = 0 then
					
					-- Indiquer qu'on ne travaille plus
					busy <= '0';
					-- Ecrire le registre sur la sortie
					dout <= reg;
					-- Mettre etat à idle
					etat <= "00";
				end if;
			end if;
		end if;
	end process;

end behavioral_3;
