library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity MasterOpl is
  port ( rst : in std_logic;
         clk : in std_logic;
         en : in std_logic;
         v1 : in std_logic_vector (7 downto 0);
         v2 : in std_logic_vector(7 downto 0);
         miso : in std_logic;
         ss   : out std_logic;
         sclk : out std_logic;
         mosi : out std_logic;
         val_nand : out std_logic_vector (7 downto 0);
         val_nor : out std_logic_vector (7 downto 0);
         val_xor : out std_logic_vector (7 downto 0);
         busy : out std_logic);
end MasterOpl;

architecture behavior of MasterOpl is

  component er_1octet
    port ( rst : in std_logic;
           clk : in std_logic;
           en : in std_logic;
           din : in std_logic_vector (7 downto 0);
           miso : in std_logic;
           sclk : out std_logic;
           mosi : out std_logic;
           dout : out std_logic_vector (7 downto 0);
           busy : out std_logic);
  end component;
  
    --Inputs
  signal reset : std_logic := '0';
  signal en_er : std_logic := '0';
  signal din_er : std_logic_vector(7 downto 0) := (others => '0');


  --Outputs
  signal busy_er : std_logic;
  signal dout_er : std_logic_vector(7 downto 0);
  
  -- types
  type t_etat is (idle, waiting, octet_envoye);
        
  -- signaux internes   
  signal etat : t_etat;

begin

  m : er_1octet
  port map ( clk => clk,
             rst => rst,
             en => en_er,
             busy => busy_er,
             din => din_er,
             dout => dout_er,
             sclk => sclk,
             miso => miso,
             mosi => mosi
           );

	process(clk,reset)
	-- variables 
    variable cpt_wait : natural;
    variable cpt_octet : natural;
	 
	begin
    if(rst = '0') then
      cpt_octet := 0;
      en_er <= '0';
      din_er <= (others => 'U');
      etat <= idle;
		busy <= '0';
		ss <= '1';
		
    elsif(rising_edge(clk)) then

      case  etat is

        when idle =>
          if (en = '1') then
				-- attente de 20 cycles de l'horloge
				-- avant le premier octet
				cpt_wait := 20;
				cpt_octet := 3;
				etat <= waiting;
				busy <= '1';
				ss <= '0';
			end if;
			 
        when waiting => 
          cpt_wait := cpt_wait - 1;
          if(cpt_wait = 0) then
            -- on a attendu assez longtemps

            -- valeur de l'octet envoyé
            if(cpt_octet = 3) then
              din_er <= v1;
            else
              din_er <= v2;
            end if;
            -- on lance l'échange d'un octet en activant
            -- le sous-composant
            en_er <= '1';
            etat <= octet_envoye;
          end if;

        when octet_envoye => 
          -- on rabaisse l'ordre passé au sous-composant
          en_er <= '0';
          -- din_er <= (others => 'U');

          if(en_er = '0' and busy_er ='0') then
            -- échange d'1 octet terminé
				
            -- on prépare la suite
				case cpt_octet is
					
					when 3 =>
					-- On réceptionne l'octet transmit par l'esclave
					val_nand <= dout_er;
				   -- il reste des octets à transmettre
               cpt_octet := cpt_octet - 1;
               -- attente de 5 cycles entre les deux octets
               cpt_wait := 5;
               etat <= waiting;
					
					when 2 =>
					-- On réceptionne l'octet transmit par l'esclave
					val_nor <= dout_er;
				   -- il reste des octets à transmettre
               cpt_octet := cpt_octet - 1;
               -- attente de 5 cycles entre les deux octets
               cpt_wait := 5;
               etat <= waiting;
					
					when 1 =>
					-- Finito pipo
					val_xor <= dout_er;
				   -- il reste des octets à transmettre
               etat <= idle;
					busy <= '0';
					ss <= '1';
					
					when others =>
						null;
            end case;

          end if;

      end case;
		
	end if;

  end process;
	
end behavior;
